@@ -5,9 +5,37 @@
 import sys
 import types
 import operator
+try:
+    from future_builtins import filter, map, zip
+except ImportError:
+    from builtins import filter, map, zip  # @UnusedImport
+try:
+    import unittest2 as unittest
+except ImportError:
+    import unittest  # @UnusedImport
+try:
+    from collections import OrderedDict
+except  ImportError:
+    from ordereddict import OrderedDict  # @UnusedImport
+try:
+    from thread import get_ident
+except ImportError:
+    try:
+        from dummy_thread import get_ident
+    except ImportError:
+        from _thread import get_ident  # @UnusedImport
+try:
+    from __builtin__ import intern
+except ImportError:
+    from sys import intern  # @UnusedImport
+try:
+    from cPickle import loads as ld, dumps, HIGHEST_PROTOCOL
+except ImportError:
+    from pickle import loads as ld, dumps, HIGHEST_PROTOCOL  # @UnusedImport
 
 __author__ = 'Benjamin Peterson <benjamin@python.org>'
 __version__ = '1.1.0'
+
 # True if we are running on Python 3.
 PY3 = sys.version_info[0] == 3
 if PY3:
@@ -16,7 +44,6 @@ if PY3:
     classes = type,
     native = texts = str
     binaries = bytes
-
     MAXSIZE = sys.maxsize
 else:
     strings = basestring,
@@ -124,30 +151,12 @@ _moved_attributes = [
         'zip_longest', 'itertools', 'itertools', 'izip_longest', 'zip_longest',
     ),
     MovedAttribute('cStringIO', 'cStringIO', 'io', 'StringIO'),
-    MovedAttribute('filter', 'itertools', 'builtins', 'ifilter', 'filter'),
-    MovedAttribute('map', 'itertools', 'builtins', 'imap', 'map'),
     MovedAttribute('reload_module', '__builtin__', 'imp', 'reload'),
     MovedAttribute('reduce', '__builtin__', 'functools'),
     MovedAttribute('StringIO', 'StringIO', 'io'),
     MovedAttribute('xrange', '__builtin__', 'builtins', 'xrange', 'range'),
-    MovedAttribute('zip', 'itertools', 'builtins', 'izip', 'zip'),
     MovedAttribute('parsedate_tz', 'rfc822', 'email.utils', 'parsedate_tz'),
     MovedAttribute('formatdate', 'rfc822', 'email.utils', 'formatdate'),
-    MovedAttribute('xrange', '__builtin__', 'builtins', 'xrange', 'range'),
-#    MovedAttribute(
-#        'read_plist',
-#        'plistlib',
-#        'plistlib',
-#        'readPlistFromString',
-#        'readPlistFromBytes',
-#    ),
-#    MovedAttribute(
-#        'write_plist',
-#        'plistlib',
-#        'plistlib',
-#        'writePlistFromString',
-#        'writePlistFromBytes',
-#    ),
     MovedAttribute('parse_qs', 'cgi', 'urllib.parse', 'parse_qs'),
     MovedAttribute('urlencode', 'urllib', 'urllib.parse', 'urlencode'),
     MovedModule('builtins', '__builtin__'),
@@ -366,3 +375,103 @@ _add_doc(reraise, '''Reraise an exception.''')
 def with_metaclass(meta, base=object):
     '''Create a base class with a metaclass.'''
     return meta('NewBase', (base,), {})
+
+
+class port(object):
+
+    '''python 2/3 helper'''
+
+    # is python 3?
+    PY3 = PY3
+    # types
+    BINARY = binaries
+    CLASS = classes
+    INTEGER = integers
+    MAXSIZE = MAXSIZE
+    STRING = strings
+    UNICODE = texts
+    # classes
+    BytesIO = BytesIO
+    StringIO = StringIO
+    # character data
+    b = staticmethod(b)
+    int2byte = staticmethod(int2byte)
+    u = staticmethod(u)
+    # dictionary
+    items = staticmethod(items)
+    keys = staticmethod(keys)
+    values = staticmethod(values)
+    # iterables
+    iterator = staticmethod(advance_iterator)
+    # classes
+    metaclass = staticmethod(with_metaclass)
+    # methods
+    code = staticmethod(function_code)
+    defaults = staticmethod(function_defaults)
+    method_function = staticmethod(method_function)
+    method_self = staticmethod(method_self)
+    unbound = staticmethod(get_unbound_function)
+    # exception
+    reraise = staticmethod(reraise)
+
+    @classmethod
+    def isbinary(cls, value):
+        '''is binary?'''
+        return isinstance(value, cls.BINARY)
+
+    @classmethod
+    def isclass(cls, value):
+        '''is class?'''
+        return isinstance(value, cls.CLASS)
+
+    @classmethod
+    def iscall(cls, value):
+        '''is callable?'''
+        return callable(value)
+
+    @classmethod
+    def isgtemax(cls, value):
+        '''greater than max size?'''
+        return value > cls.MAXSIZE
+
+    @classmethod
+    def isinteger(cls, value):
+        '''is integer?'''
+        return isinstance(value, cls.INTEGER)
+
+    @classmethod
+    def isltemax(cls, value):
+        '''less than max size?'''
+        return value < cls.MAXSIZE
+
+    @classmethod
+    def isstring(cls, value):
+        '''is string'''
+        return isinstance(value, cls.STRING)
+
+    @classmethod
+    def isunicode(cls, value):
+        '''is text?'''
+        return isinstance(value, cls.UNICODE)
+
+    @staticmethod
+    def printf(*args, **kw):
+        '''print output'''
+        return printf(*args, **kw)
+
+isbinary = port.isbinary
+isstring = port.isstring
+isunicode = port.isunicode
+
+
+def tounicode(thing, encoding='utf-8', errors='strict'):
+    return (
+        thing.decode(encoding, errors) if isbinary(thing) else
+        texts(texts(thing).encode(encoding, errors), encoding, errors)
+    )
+
+
+def tobytes(thing, encoding='utf-8', errors='strict'):
+    return (
+        texts(thing).encode(encoding, errors) if not isbinary(thing) else thing
+    )
